"""Google Workspace Automation helper library.

This library provides a set of helper functions to automate common tasks in
Google Workspace. It is not a complete API wrapper, but rather a set of
functions that use the Google APIs to perform common tasks.
"""
